@@ -1,5 +1,5 @@
 /**
- * Copyright 2012,2013  Nick Galbreath
+ * Copyright 2012,2016  Nick Galbreath
  * nickg@client9.com
  * BSD License -- see COPYING.txt for details
  *
@@ -14,7 +14,14 @@
 #include <assert.h>
 #include <stddef.h>
 
-#define LIBINJECTION_VERSION "3.8.0"
+#include "libinjection.h"
+#include "libinjection_sqli.h"
+#include "libinjection_sqli_data.h"
+
+#define LIBINJECTION_VERSION "3.9.2"
+
+#define LIBINJECTION_SQLI_TOKEN_SIZE  sizeof(((stoken_t*)(0))->val)
+#define LIBINJECTION_SQLI_MAX_TOKENS  5
 
 #ifndef TRUE
 #define TRUE 1
@@ -37,40 +44,38 @@
 #define FOLD_DEBUG
 #endif
 
-#include "libinjection_sqli_data.h"
-
 /*
  * not making public just yet
  */
 typedef enum {
-    TYPE_NONE        = 0,
-    TYPE_KEYWORD     = (int)'k',
-    TYPE_UNION       = (int)'U',
-    TYPE_GROUP       = (int)'B',
-    TYPE_EXPRESSION  = (int)'E',
-    TYPE_SQLTYPE     = (int)'t',
-    TYPE_FUNCTION    = (int)'f',
-    TYPE_BAREWORD    = (int)'n',
-    TYPE_NUMBER      = (int)'1',
-    TYPE_VARIABLE    = (int)'v',
-    TYPE_STRING      = (int)'s',
-    TYPE_OPERATOR    = (int)'o',
-    TYPE_LOGIC_OPERATOR = (int)'&',
-    TYPE_COMMENT     = (int)'c',
-    TYPE_COLLATE     = (int)'A',
-    TYPE_LEFTPARENS  = (int)'(',
-    TYPE_RIGHTPARENS = (int)')',  /* not used? */
-    TYPE_LEFTBRACE   = (int)'{',
-    TYPE_RIGHTBRACE  = (int)'}',
-    TYPE_DOT         = (int)'.',
-    TYPE_COMMA       = (int)',',
-    TYPE_COLON       = (int)':',
-    TYPE_SEMICOLON   = (int)';',
-    TYPE_TSQL        = (int)'T',  /* TSQL start */
-    TYPE_UNKNOWN     = (int)'?',
-    TYPE_EVIL        = (int)'X',  /* unparsable, abort  */
-    TYPE_FINGERPRINT = (int)'F',  /* not really a token */
-    TYPE_BACKSLASH   = (int)'\\'
+    TYPE_NONE        = 0
+    , TYPE_KEYWORD     = (int)'k'
+    , TYPE_UNION       = (int)'U'
+    , TYPE_GROUP       = (int)'B'
+    , TYPE_EXPRESSION  = (int)'E'
+    , TYPE_SQLTYPE     = (int)'t'
+    , TYPE_FUNCTION    = (int)'f'
+    , TYPE_BAREWORD    = (int)'n'
+    , TYPE_NUMBER      = (int)'1'
+    , TYPE_VARIABLE    = (int)'v'
+    , TYPE_STRING      = (int)'s'
+    , TYPE_OPERATOR    = (int)'o'
+    , TYPE_LOGIC_OPERATOR = (int)'&'
+    , TYPE_COMMENT     = (int)'c'
+    , TYPE_COLLATE     = (int)'A'
+    , TYPE_LEFTPARENS  = (int)'('
+    , TYPE_RIGHTPARENS = (int)')'  /* not used? */
+    , TYPE_LEFTBRACE   = (int)'{'
+    , TYPE_RIGHTBRACE  = (int)'}'
+    , TYPE_DOT         = (int)'.'
+    , TYPE_COMMA       = (int)','
+    , TYPE_COLON       = (int)':'
+    , TYPE_SEMICOLON   = (int)';'
+    , TYPE_TSQL        = (int)'T'  /* TSQL start */
+    , TYPE_UNKNOWN     = (int)'?'
+    , TYPE_EVIL        = (int)'X'  /* unparsable, abort  */
+    , TYPE_FINGERPRINT = (int)'F'  /* not really a token */
+    , TYPE_BACKSLASH   = (int)'\\'
 } sqli_token_types;
 
 /**
@@ -107,15 +112,11 @@ memchr2(const char *haystack, size_t haystack_len, char c0, char c1)
     }
 
     while (cur < last) {
-        if (cur[0] == c0) {
-            if (cur[1] == c1) {
-                return cur;
-            } else {
-                cur += 2; /* (c0 == c1) ? 1 : 2; */
-            }
-        } else {
-            cur += 1;
+        /* safe since cur < len - 1 always */
+        if (cur[0] == c0 && cur[1] == c1) {
+            return cur;
         }
+        cur += 1;
     }
 
     return NULL;
@@ -186,11 +187,11 @@ static int char_is_white(char ch) {
     /* ' '  space is 0x32
        '\t  0x09 \011 horizontal tab
        '\n' 0x0a \012 new line
-       '\v' 0x0b \013 verical tab
+       '\v' 0x0b \013 vertical tab
        '\f' 0x0c \014 new page
        '\r' 0x0d \015 carriage return
             0x00 \000 null (oracle)
-            0xa0 \240 is latin1
+            0xa0 \240 is Latin-1
     */
     return strchr(" \t\n\v\f\r\240\000", ch) != NULL;
 }
@@ -289,7 +290,7 @@ static void st_clear(stoken_t * st)
 static void st_assign_char(stoken_t * st, const char stype, size_t pos, size_t len,
                            const char value)
 {
-    /* done to elimiate unused warning */
+    /* done to eliminate unused warning */
     (void)len;
     st->type = (char) stype;
     st->pos = pos;
@@ -392,12 +393,12 @@ static size_t parse_eol_comment(struct libinjection_sqli_state * sf)
         st_assign(sf->current, TYPE_COMMENT, pos, slen - pos, cs + pos);
         return slen;
     } else {
-        st_assign(sf->current, TYPE_COMMENT, pos, endpos - cs - pos, cs + pos);
-        return (endpos - cs) + 1;
+        st_assign(sf->current, TYPE_COMMENT, pos, (size_t)(endpos - cs) - pos, cs + pos);
+        return (size_t)((endpos - cs) + 1);
     }
 }
 
-/** In Ansi mode, hash is an operator
+/** In ANSI mode, hash is an operator
  *  In MYSQL mode, it's a EOL comment like '--'
  */
 static size_t parse_hash(struct libinjection_sqli_state * sf)
@@ -511,7 +512,7 @@ static size_t parse_slash(struct libinjection_sqli_state * sf)
         /* till end of line */
         clen = slen - pos;
     } else {
-        clen = (ptr + 2) - cur;
+        clen = (size_t)(ptr + 2 - cur);
     }
 
     /*
@@ -524,7 +525,7 @@ static size_t parse_slash(struct libinjection_sqli_state * sf)
      *  are an automatic black ban!
      */
 
-    if (memchr2(cur + 2, ptr - (cur + 1), '/', '*') !=  NULL) {
+    if (memchr2(cur + 2, (size_t)(ptr - (cur + 1)), '/', '*') !=  NULL) {
         ctype = TYPE_EVIL;
     } else if (is_mysql_comment(cs, slen, pos)) {
         ctype = TYPE_EVIL;
@@ -621,7 +622,7 @@ static size_t is_double_delim_escaped(const char* cur,  const char* end)
     return  ((cur + 1) < end) && *(cur+1) == *cur;
 }
 
-/* Look forward for doubling of deliminter
+/* Look forward for doubling of delimiter
  *
  * case 'foo''bar' --> foo''bar
  *
@@ -667,20 +668,20 @@ static size_t parse_string_core(const char *cs, const size_t len, size_t pos,
             /* keep going, move ahead one character */
             qpos =
                 (const char *) memchr((const void *) (qpos + 1), delim,
-                                      (cs + len) - (qpos + 1));
+                                      (size_t)((cs + len) - (qpos + 1)));
             continue;
         } else if (is_double_delim_escaped(qpos, cs + len)) {
             /* keep going, move ahead two characters */
             qpos =
                 (const char *) memchr((const void *) (qpos + 2), delim,
-                                      (cs + len) - (qpos + 2));
+                                      (size_t)((cs + len) - (qpos + 2)));
             continue;
         } else {
             /* hey it's a normal string */
             st_assign(st, TYPE_STRING, pos + offset,
-                      qpos - (cs + pos + offset), cs + pos + offset);
+                      (size_t)(qpos - (cs + pos + offset)), cs + pos + offset);
             st->str_close = delim;
-            return qpos - cs + 1;
+            return (size_t)(qpos - cs + 1);
         }
     }
 }
@@ -736,7 +737,7 @@ static size_t parse_ustring(struct libinjection_sqli_state * sf)
     }
 }
 
-static size_t parse_qstring_core(struct libinjection_sqli_state * sf, int offset)
+static size_t parse_qstring_core(struct libinjection_sqli_state * sf, size_t offset)
 {
     char ch;
     const char *strend;
@@ -779,10 +780,10 @@ static size_t parse_qstring_core(struct libinjection_sqli_state * sf, int offset
         sf->current->str_close = CHAR_NULL;
         return slen;
     } else {
-        st_assign(sf->current, TYPE_STRING, pos + 3, strend - cs - pos -  3, cs + pos + 3);
+        st_assign(sf->current, TYPE_STRING, pos + 3, (size_t)(strend - cs) - pos -  3, cs + pos + 3);
         sf->current->str_open = 'q';
         sf->current->str_close = 'q';
-        return (strend - cs) + 2;
+        return (size_t)(strend - cs + 2);
     }
 }
 
@@ -837,7 +838,7 @@ static size_t parse_bstring(struct libinjection_sqli_state *sf)
 
 /*
  * hex literal string
- * re: [XX]'[0123456789abcdefABCDEF]*'
+ * re: [xX]'[0123456789abcdefABCDEF]*'
  * mysql has requirement of having EVEN number of chars,
  *  but pgsql does not
  */
@@ -878,8 +879,8 @@ static size_t parse_bword(struct libinjection_sqli_state * sf)
         st_assign(sf->current, TYPE_BAREWORD, pos, sf->slen - pos, cs + pos);
         return sf->slen;
     } else {
-        st_assign(sf->current, TYPE_BAREWORD, pos, (endptr - cs) - pos + 1, cs + pos);
-        return (endptr - cs) + 1;
+        st_assign(sf->current, TYPE_BAREWORD, pos, (size_t)(endptr - cs) - pos + 1, cs + pos);
+        return (size_t)((endptr - cs) + 1);
     }
 }
 
@@ -1042,10 +1043,11 @@ static size_t parse_money(struct libinjection_sqli_state *sf)
                 sf->current->str_close = CHAR_NULL;
                 return slen;
             } else {
-                st_assign(sf->current, TYPE_STRING, pos + 2, strend - (cs + pos + 2), cs + pos + 2);
+                st_assign(sf->current, TYPE_STRING, pos + 2,
+                          (size_t)(strend - (cs + pos + 2)), cs + pos + 2);
                 sf->current->str_open = '$';
                 sf->current->str_close = '$';
-                return strend - cs + 2;
+                return (size_t)(strend - cs + 2);
             }
         } else {
             /* ok it's not a number or '$$', but maybe it's pgsql "$ quoted strings" */
@@ -1066,7 +1068,7 @@ static size_t parse_money(struct libinjection_sqli_state *sf)
             /* we have $foobar$ ... find it again */
             strend = my_memmem(cs+xlen+2, slen - (pos+xlen+2), cs + pos, xlen+2);
 
-            if (strend == NULL) {
+            if (strend == NULL || ((size_t)(strend - cs) < (pos+xlen+2))) {
                 /* fell off edge */
                 st_assign(sf->current, TYPE_STRING, pos+xlen+2, slen - pos - xlen - 2, cs+pos+xlen+2);
                 sf->current->str_open = '$';
@@ -1074,10 +1076,11 @@ static size_t parse_money(struct libinjection_sqli_state *sf)
                 return slen;
             } else {
                 /* got one */
-                st_assign(sf->current, TYPE_STRING, pos+xlen+2, strend - (cs + pos + xlen + 2), cs+pos+xlen+2);
+                st_assign(sf->current, TYPE_STRING, pos+xlen+2,
+                          (size_t)(strend - (cs + pos + xlen + 2)), cs+pos+xlen+2);
                 sf->current->str_open = '$';
                 sf->current->str_close = '$';
-                return (strend + xlen + 2) - cs;
+                return (size_t)((strend + xlen + 2) - cs);
             }
         }
     } else if (xlen == 1 && cs[pos + 1] == '.') {
@@ -1097,7 +1100,6 @@ static size_t parse_number(struct libinjection_sqli_state * sf)
     const char *cs = sf->s;
     const size_t slen = sf->slen;
     size_t pos = sf->pos;
-    int have_dot = 0;
     int have_e = 0;
     int have_exp = 0;
 
@@ -1129,7 +1131,6 @@ static size_t parse_number(struct libinjection_sqli_state * sf)
     }
 
     if (pos < slen && cs[pos] == '.') {
-        have_dot = 1;
         pos += 1;
         while (pos < slen && ISDIGIT(cs[pos])) {
             pos += 1;
@@ -1178,7 +1179,7 @@ static size_t parse_number(struct libinjection_sqli_state * sf)
         }
     }
 
-    if (have_dot == 1 && have_e == 1 && have_exp == 0) {
+    if (have_e == 1 && have_exp == 0) {
         /* very special form of
          * "1234.e"
          * "10.10E"
@@ -1232,32 +1233,16 @@ int libinjection_sqli_tokenize(struct libinjection_sqli_state * sf)
         /*
          * get current character
          */
-        const unsigned char ch = (unsigned int) (s[*pos]);
+        const unsigned char ch = (unsigned char) (s[*pos]);
 
         /*
-         * if not ascii, then continue...
-         *   actually probably need to just assuming
-         *   it's a string
+         * look up the parser, and call it
+         *
+         * Porting Note: this is mapping of char to function
+         *   charparsers[ch]()
          */
-        if (ch > 127) {
+        fnptr = char_parse_map[ch];
 
-            /* 160 or 0xA0 or octal 240 is "latin1 non-breaking space"
-             * but is treated as a space in mysql.
-             */
-            if (ch == 160) {
-                fnptr = parse_white;
-            } else {
-                fnptr = parse_word;
-            }
-        } else {
-            /*
-             * look up the parser, and call it
-             *
-             * Porting Note: this is mapping of char to function
-             *   charparsers[ch]()
-             */
-            fnptr = char_parse_map[ch];
-        }
         *pos = (*fnptr) (sf);
 
         /*
@@ -1342,16 +1327,22 @@ static int syntax_merge_words(struct libinjection_sqli_state * sf,stoken_t * a,
          a->type == TYPE_UNION ||
          a->type == TYPE_FUNCTION ||
          a->type == TYPE_EXPRESSION ||
+         a->type == TYPE_TSQL ||
          a->type == TYPE_SQLTYPE)) {
-        return CHAR_NULL;
+        return FALSE;
     }
 
-    if (b->type != TYPE_KEYWORD  && b->type != TYPE_BAREWORD &&
-        b->type != TYPE_OPERATOR && b->type != TYPE_SQLTYPE &&
-        b->type != TYPE_LOGIC_OPERATOR &&
-        b->type != TYPE_FUNCTION &&
-        b->type != TYPE_UNION    && b->type != TYPE_EXPRESSION) {
-        return CHAR_NULL;
+    if (!
+        (b->type == TYPE_KEYWORD ||
+         b->type == TYPE_BAREWORD ||
+         b->type == TYPE_OPERATOR ||
+         b->type == TYPE_UNION ||
+         b->type == TYPE_FUNCTION ||
+         b->type == TYPE_EXPRESSION ||
+         b->type == TYPE_TSQL ||
+         b->type == TYPE_SQLTYPE ||
+         b->type == TYPE_LOGIC_OPERATOR)) {
+        return FALSE;
     }
 
     sz1 = a->len;
@@ -1367,7 +1358,6 @@ static int syntax_merge_words(struct libinjection_sqli_state * sf,stoken_t * a,
     tmp[sz1] = ' ';
     memcpy(tmp + sz1 + 1, b->val, sz2);
     tmp[sz3] = CHAR_NULL;
-
     ch = sf->lookup(sf, LOOKUP_WORD, tmp, sz3);
 
     if (ch != CHAR_NULL) {
@@ -1399,7 +1389,10 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
     sf->current = &(sf->tokenvec[0]);
     while (more) {
         more = libinjection_sqli_tokenize(sf);
-        if ( ! (sf->current->type == TYPE_COMMENT || sf->current->type == TYPE_LEFTPARENS || st_is_unary_op(sf->current))) {
+        if ( ! (sf->current->type == TYPE_COMMENT ||
+                sf->current->type == TYPE_LEFTPARENS ||
+                sf->current->type == TYPE_SQLTYPE ||
+                st_is_unary_op(sf->current))) {
             break;
         }
     }
@@ -1440,19 +1433,24 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
                     sf->tokenvec[2].type == TYPE_COMMA &&
                     sf->tokenvec[3].type == TYPE_LEFTPARENS &&
                     sf->tokenvec[4].type == TYPE_NUMBER
+                    ) ||
+                (
+                    sf->tokenvec[0].type == TYPE_BAREWORD &&
+                    sf->tokenvec[1].type == TYPE_RIGHTPARENS &&
+                    sf->tokenvec[2].type == TYPE_OPERATOR &&
+                    sf->tokenvec[3].type == TYPE_LEFTPARENS &&
+                    sf->tokenvec[4].type == TYPE_BAREWORD
                     )
                 )
             {
                 if (pos > LIBINJECTION_SQLI_MAX_TOKENS) {
-                    //printf("TRUNCATE pos = %lu vs. %lu\n", pos, LIBINJECTION_SQLI_MAX_TOKENS);
-                    st_copy(&(sf->tokenvec[1]), &(sf->tokenvec[LIBINJECTION_SQLI_MAX_TOKENS]));
+		    st_copy(&(sf->tokenvec[1]), &(sf->tokenvec[LIBINJECTION_SQLI_MAX_TOKENS]));
                     pos = 2;
                     left = 0;
                 } else {
-                    //printf("HEREIAM\n");
                     pos = 1;
                     left = 0;
-                } 
+                }
             }
         }
 
@@ -1498,24 +1496,15 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
             pos -= 1;
             sf->stats_folds += 1;
             continue;
-        } else if (sf->tokenvec[left].type == TYPE_SEMICOLON &&
-                   sf->tokenvec[left+1].type == TYPE_FUNCTION &&
-                   cstrcasecmp("IF", sf->tokenvec[left+1].val, sf->tokenvec[left+1].len) == 0) {
-            /* IF is normally a function, except in Transact-SQL where it can be used as a
-             * standalone control flow operator, e.g. ; IF 1=1 ...
-             * if found after a semicolon, convert from 'f' type to 'T' type
-             */
-            sf->tokenvec[left+1].type = TYPE_TSQL;
-            left += 2;
-            continue; /* reparse everything, but we probably can advance left, and pos */
-        } else if ((sf->tokenvec[left].type ==TYPE_OPERATOR ||
-                    sf->tokenvec[left].type ==TYPE_LOGIC_OPERATOR) &&
-                   st_is_unary_op(&sf->tokenvec[left+1])) {
+        } else if ((sf->tokenvec[left].type == TYPE_OPERATOR ||
+                    sf->tokenvec[left].type == TYPE_LOGIC_OPERATOR) &&
+                   (st_is_unary_op(&sf->tokenvec[left+1]) ||
+                    sf->tokenvec[left+1].type == TYPE_SQLTYPE)) {
             pos -= 1;
             sf->stats_folds += 1;
             left = 0;
             continue;
-        } else if (sf->tokenvec[left].type ==TYPE_LEFTPARENS &&
+        } else if (sf->tokenvec[left].type == TYPE_LEFTPARENS &&
                    st_is_unary_op(&sf->tokenvec[left+1])) {
             pos -= 1;
             sf->stats_folds += 1;
@@ -1530,9 +1519,22 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
                 left -= 1;
             }
             continue;
+        } else if (sf->tokenvec[left].type == TYPE_SEMICOLON &&
+                   sf->tokenvec[left+1].type == TYPE_FUNCTION &&
+		   (sf->tokenvec[left+1].val[0] == 'I' ||
+		    sf->tokenvec[left+1].val[0] == 'i' ) &&
+		   (sf->tokenvec[left+1].val[1] == 'F' ||
+                    sf->tokenvec[left+1].val[1] == 'f' )) {
+            /* IF is normally a function, except in Transact-SQL where it can be used as a
+             * standalone control flow operator, e.g. ; IF 1=1 ...
+             * if found after a semicolon, convert from 'f' type to 'T' type
+             */
+            sf->tokenvec[left+1].type = TYPE_TSQL;
+            /* left += 2; */
+            continue; /* reparse everything, but we probably can advance left, and pos */
         } else if ((sf->tokenvec[left].type == TYPE_BAREWORD || sf->tokenvec[left].type == TYPE_VARIABLE) &&
                    sf->tokenvec[left+1].type == TYPE_LEFTPARENS && (
-                       /* TSQL functions but common enough to be collumn names */
+                       /* TSQL functions but common enough to be column names */
                        cstrcasecmp("USER_ID", sf->tokenvec[left].val, sf->tokenvec[left].len) == 0 ||
                        cstrcasecmp("USER_NAME", sf->tokenvec[left].val, sf->tokenvec[left].len) == 0 ||
 
@@ -1555,7 +1557,7 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
 
             /* pos is the same
              * other conversions need to go here... for instance
-             * password CAN be a function, coalese CAN be a function
+             * password CAN be a function, coalesce CAN be a function
              */
             sf->tokenvec[left].type = TYPE_FUNCTION;
             continue;
@@ -1587,9 +1589,20 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
              *  "foo" = LIKE(1,2)
              */
             continue;
+        } else if ((sf->tokenvec[left].type == TYPE_OPERATOR) && (
+                       cstrcasecmp("LIKE", sf->tokenvec[left].val, sf->tokenvec[left].len) == 0 ||
+                       cstrcasecmp("NOT LIKE", sf->tokenvec[left].val, sf->tokenvec[left].len) == 0)) {
+            if (sf->tokenvec[left+1].type == TYPE_LEFTPARENS) {
+                /* SELECT LIKE(...
+                 * it's a function
+                 */
+                sf->tokenvec[left].type = TYPE_FUNCTION;
+            }
         } else if (sf->tokenvec[left].type == TYPE_SQLTYPE &&
                    (sf->tokenvec[left+1].type == TYPE_BAREWORD ||
                     sf->tokenvec[left+1].type == TYPE_NUMBER ||
+                    sf->tokenvec[left+1].type == TYPE_SQLTYPE ||
+                    sf->tokenvec[left+1].type == TYPE_LEFTPARENS ||
                     sf->tokenvec[left+1].type == TYPE_FUNCTION ||
                     sf->tokenvec[left+1].type == TYPE_VARIABLE ||
                     sf->tokenvec[left+1].type == TYPE_STRING))  {
@@ -1606,6 +1619,7 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
              */
             if (strchr(sf->tokenvec[left+1].val, '_') != NULL) {
                 sf->tokenvec[left+1].type = TYPE_SQLTYPE;
+                left = 0;
             }
         } else if (sf->tokenvec[left].type == TYPE_BACKSLASH) {
             if (st_is_arithmetic_op(&(sf->tokenvec[left+1]))) {
@@ -1622,31 +1636,52 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
         } else if (sf->tokenvec[left].type == TYPE_LEFTPARENS &&
                    sf->tokenvec[left+1].type == TYPE_LEFTPARENS) {
             pos -= 1;
+            left = 0;
             sf->stats_folds += 1;
             continue;
         } else if (sf->tokenvec[left].type == TYPE_RIGHTPARENS &&
                    sf->tokenvec[left+1].type == TYPE_RIGHTPARENS) {
             pos -= 1;
-            sf->stats_folds += 1;
-            continue;
-        } else if (sf->tokenvec[left].type == TYPE_SQLTYPE &&
-                   sf->tokenvec[left+1].type == TYPE_SQLTYPE) {
-            pos -= 1;
+            left = 0;
             sf->stats_folds += 1;
             continue;
         } else if (sf->tokenvec[left].type == TYPE_LEFTBRACE &&
                    sf->tokenvec[left+1].type == TYPE_BAREWORD) {
+
+            /*
+             * MySQL Degenerate case --
+             *
+             *   select { ``.``.id };  -- valid !!!
+             *   select { ``.``.``.id };  -- invalid
+             *   select ``.``.id; -- invalid
+             *   select { ``.id }; -- invalid
+             *
+             * so it appears {``.``.id} is a magic case
+             * I suspect this is "current database, current table, field id"
+             *
+             * The folding code can't look at more than 3 tokens, and
+             * I don't want to make two passes.
+             *
+             * Since "{ ``" so rare, we are just going to blacklist it.
+             *
+             * Highly likely this will need revisiting!
+             *
+             * CREDIT @rsalgado 2013-11-25
+             */
+            if (sf->tokenvec[left+1].len == 0) {
+                sf->tokenvec[left+1].type = TYPE_EVIL;
+                return (int)(left+2);
+            }
             /* weird ODBC / MYSQL  {foo expr} --> expr
              * but for this rule we just strip away the "{ foo" part
              */
-            if (left > 0) {
-                left -= 1;
-            }
+            left = 0;
             pos -= 2;
             sf->stats_folds += 2;
             continue;
         } else if (sf->tokenvec[left+1].type == TYPE_RIGHTBRACE) {
             pos -= 1;
+            left = 0;
             sf->stats_folds += 1;
             continue;
         }
@@ -1681,18 +1716,18 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
             sf->tokenvec[left+1].type == TYPE_OPERATOR &&
             sf->tokenvec[left+2].type == TYPE_NUMBER) {
             pos -= 2;
+            left = 0;
             continue;
         } else if (sf->tokenvec[left].type == TYPE_OPERATOR &&
                    sf->tokenvec[left+1].type != TYPE_LEFTPARENS &&
                    sf->tokenvec[left+2].type == TYPE_OPERATOR) {
-            if (left > 0) {
-                left -= 1;
-            }
+            left = 0;
             pos -= 2;
             continue;
         } else if (sf->tokenvec[left].type == TYPE_LOGIC_OPERATOR &&
                    sf->tokenvec[left+2].type == TYPE_LOGIC_OPERATOR) {
             pos -= 2;
+            left = 0;
             continue;
         } else if (sf->tokenvec[left].type == TYPE_VARIABLE &&
                    sf->tokenvec[left+1].type == TYPE_OPERATOR &&
@@ -1700,6 +1735,7 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
                     sf->tokenvec[left+2].type == TYPE_NUMBER ||
                     sf->tokenvec[left+2].type == TYPE_BAREWORD)) {
             pos -= 2;
+            left = 0;
             continue;
         } else if ((sf->tokenvec[left].type == TYPE_BAREWORD ||
                     sf->tokenvec[left].type == TYPE_NUMBER ) &&
@@ -1707,6 +1743,7 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
                    (sf->tokenvec[left+2].type == TYPE_NUMBER ||
                     sf->tokenvec[left+2].type == TYPE_BAREWORD)) {
             pos -= 2;
+            left = 0;
             continue;
         } else if ((sf->tokenvec[left].type == TYPE_BAREWORD ||
                     sf->tokenvec[left].type == TYPE_NUMBER ||
@@ -1716,6 +1753,7 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
                    streq(sf->tokenvec[left+1].val, "::") &&
                    sf->tokenvec[left+2].type == TYPE_SQLTYPE) {
             pos -= 2;
+            left = 0;
             sf->stats_folds += 2;
             continue;
         } else if ((sf->tokenvec[left].type == TYPE_BAREWORD ||
@@ -1728,9 +1766,7 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
                     sf->tokenvec[left+2].type == TYPE_STRING ||
                     sf->tokenvec[left+2].type == TYPE_VARIABLE)) {
             pos -= 2;
-            if (left > 0) {
-                left -= 1;
-            }
+            left = 0;
             continue;
         } else if ((sf->tokenvec[left].type == TYPE_EXPRESSION ||
                     sf->tokenvec[left].type == TYPE_GROUP ||
@@ -1742,6 +1778,7 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
              */
             st_copy(&sf->tokenvec[left+1], &sf->tokenvec[left+2]);
             pos -= 1;
+            left = 0;
             continue;
         } else if ((sf->tokenvec[left].type == TYPE_KEYWORD ||
                     sf->tokenvec[left].type == TYPE_EXPRESSION ||
@@ -1757,6 +1794,7 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
              */
             st_copy(&sf->tokenvec[left+1], &sf->tokenvec[left+2]);
             pos -= 1;
+            left = 0;
             continue;
         } else if (sf->tokenvec[left].type == TYPE_COMMA &&
                    st_is_unary_op(&sf->tokenvec[left+1]) &&
@@ -1770,9 +1808,7 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
              * "1,-1" --> "1"
              */
             st_copy(&sf->tokenvec[left+1], &sf->tokenvec[left+2]);
-            if (left > 0) {
-                left -= 1;
-            }
+            left = 0;
             /* pos is >= 3 so this is safe */
             assert(pos >= 3);
             pos -= 3;
@@ -1781,14 +1817,15 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
                    st_is_unary_op(&sf->tokenvec[left+1]) &&
                    sf->tokenvec[left+2].type == TYPE_FUNCTION) {
 
-            /* Seperate case from above since you end up with
+            /* Separate case from above since you end up with
              * 1,-sin(1) --> 1 (1)
              * Here, just do
              * 1,-sin(1) --> 1,sin(1)
-             * just remove unary opartor
+             * just remove unary operator
              */
             st_copy(&sf->tokenvec[left+1], &sf->tokenvec[left+2]);
             pos -= 1;
+            left = 0;
             continue;
         } else if ((sf->tokenvec[left].type == TYPE_BAREWORD) &&
                    (sf->tokenvec[left+1].type == TYPE_DOT) &&
@@ -1798,6 +1835,7 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
              */
             assert(pos >= 3);
             pos -= 2;
+            left = 0;
             continue;
         } else if ((sf->tokenvec[left].type == TYPE_EXPRESSION) &&
                    (sf->tokenvec[left+1].type == TYPE_DOT) &&
@@ -1805,10 +1843,23 @@ int libinjection_sqli_fold(struct libinjection_sqli_state * sf)
             /* select . `foo` --> select `foo` */
             st_copy(&sf->tokenvec[left+1], &sf->tokenvec[left+2]);
             pos -= 1;
+            left = 0;
             continue;
+        } else if ((sf->tokenvec[left].type == TYPE_FUNCTION) &&
+                   (sf->tokenvec[left+1].type == TYPE_LEFTPARENS) &&
+                   (sf->tokenvec[left+2].type != TYPE_RIGHTPARENS)) {
+            /*
+             * whats going on here
+             * Some SQL functions like USER() have 0 args
+             * if we get User(foo), then User is not a function
+             * This should be expanded since it eliminated a lot of false
+             * positives. 
+             */
+            if  (cstrcasecmp("USER", sf->tokenvec[left].val, sf->tokenvec[left].len) == 0) {
+                sf->tokenvec[left].type = TYPE_BAREWORD;
+            }
         }
 
-
         /* no folding -- assume left-most token is
            is good, now use the existing 2 tokens --
            do not get another
@@ -1906,12 +1957,6 @@ const char* libinjection_sqli_fingerprint(struct libinjection_sqli_state * sql_s
     return sql_state->fingerprint;
 }
 
-
-/**
- *
- */
-#define UNUSED(x) (void)(x)
-
 int libinjection_sqli_check_fingerprint(struct libinjection_sqli_state* sql_state)
 {
     return libinjection_sqli_blacklist(sql_state) &&
@@ -1934,7 +1979,7 @@ int libinjection_sqli_blacklist(struct libinjection_sqli_state* sql_state)
      * use minimum of 8 bytes to make sure gcc -fstack-protector
      * works correctly
      */
-    char fp2[LIBINJECTION_SQLI_MAX_TOKENS + 2 < 8 ? 8 : LIBINJECTION_SQLI_MAX_TOKENS + 2];
+    char fp2[8];
     char ch;
     size_t i;
     size_t len = strlen(sql_state->fingerprint);
@@ -1979,7 +2024,7 @@ int libinjection_sqli_blacklist(struct libinjection_sqli_state* sql_state)
 }
 
 /*
- * return TRUE if sqli, false is benign
+ * return TRUE if SQLi, false is benign
  */
 int libinjection_sqli_not_whitelist(struct libinjection_sqli_state* sql_state)
 {
@@ -1993,10 +2038,10 @@ int libinjection_sqli_not_whitelist(struct libinjection_sqli_state* sql_state)
 
     if (tlen > 1 && sql_state->fingerprint[tlen-1] == TYPE_COMMENT) {
         /*
-         * if ending comment is contains 'sp_password' then it's sqli!
-         * MS Audit log appearantly ignores anything with
-         * 'sp_password' in it. Unable to find primary refernece to
-         * this "feature" of SQL Server but seems to be known sqli
+         * if ending comment is contains 'sp_password' then it's SQLi!
+         * MS Audit log apparently ignores anything with
+         * 'sp_password' in it. Unable to find primary reference to
+         * this "feature" of SQL Server but seems to be known SQLi
          * technique
          */
         if (my_memmem(sql_state->s, sql_state->slen,
@@ -2015,7 +2060,7 @@ int libinjection_sqli_not_whitelist(struct libinjection_sqli_state* sql_state)
 
         if (sql_state->fingerprint[1] == TYPE_UNION) {
             if (sql_state->stats_tokens == 2) {
-                /* not sure why but 1U comes up in Sqli attack
+                /* not sure why but 1U comes up in SQLi attack
                  * likely part of parameter splitting/etc.
                  * lots of reasons why "1 union" might be normal
                  * input, so beep only if other SQLi things are present
@@ -2040,7 +2085,7 @@ int libinjection_sqli_not_whitelist(struct libinjection_sqli_state* sql_state)
 
         /*
          * for fingerprint like 'nc', only comments of /x are treated
-         * as SQL... ending comments of "--" and "#" are not sqli
+         * as SQL... ending comments of "--" and "#" are not SQLi
          */
         if (sql_state->tokenvec[0].type == TYPE_BAREWORD &&
             sql_state->tokenvec[1].type == TYPE_COMMENT &&
@@ -2050,7 +2095,7 @@ int libinjection_sqli_not_whitelist(struct libinjection_sqli_state* sql_state)
         }
 
         /*
-         * if '1c' ends with '/x' then it's sqli
+         * if '1c' ends with '/x' then it's SQLi
          */
         if (sql_state->tokenvec[0].type == TYPE_NUMBER &&
             sql_state->tokenvec[1].type == TYPE_COMMENT &&
@@ -2073,13 +2118,13 @@ int libinjection_sqli_not_whitelist(struct libinjection_sqli_state* sql_state)
         if (sql_state->tokenvec[0].type == TYPE_NUMBER &&
             sql_state->tokenvec[1].type == TYPE_COMMENT) {
             if (sql_state->stats_tokens > 2) {
-                /* we have some folding going on, highly likely sqli */
+                /* we have some folding going on, highly likely SQLi */
                 sql_state->reason = __LINE__;
                 return TRUE;
             }
             /*
              * we check that next character after the number is either whitespace,
-             * or '/' or a '-' ==> sqli.
+             * or '/' or a '-' ==> SQLi.
              */
             ch = sql_state->s[sql_state->tokenvec[0].len];
             if ( ch <= 32 ) {
@@ -2101,7 +2146,7 @@ int libinjection_sqli_not_whitelist(struct libinjection_sqli_state* sql_state)
         }
 
         /*
-         * detect obvious sqli scans.. many people put '--' in plain text
+         * detect obvious SQLi scans.. many people put '--' in plain text
          * so only detect if input ends with '--', e.g. 1-- but not 1-- foo
          */
         if ((sql_state->tokenvec[1].len > 2)
@@ -2137,7 +2182,7 @@ int libinjection_sqli_not_whitelist(struct libinjection_sqli_state* sql_state)
                 }
 
                 /*
-                 * not sqli
+                 * not SQLi
                  */
                 sql_state->reason = __LINE__;
                 return FALSE;
@@ -2146,8 +2191,8 @@ int libinjection_sqli_not_whitelist(struct libinjection_sqli_state* sql_state)
                    streq(sql_state->fingerprint, "1&1") ||
                    streq(sql_state->fingerprint, "1&v") ||
                    streq(sql_state->fingerprint, "1&s")) {
-            /* 'sexy and 17' not sqli
-             * 'sexy and 17<18'  sqli
+            /* 'sexy and 17' not SQLi
+             * 'sexy and 17<18'  SQLi
              */
             if (sql_state->stats_tokens == 3) {
                 sql_state->reason = __LINE__;
@@ -2191,7 +2236,7 @@ static int reparse_as_mysql(struct libinjection_sqli_state * sql_state)
 struct libinjection_sqli_token*
 libinjection_sqli_get_token(struct libinjection_sqli_state * sql_state, int i)
 {
-    if (i < 0 || i >  LIBINJECTION_SQLI_MAX_TOKENS) {
+    if (i < 0 || i > (int)LIBINJECTION_SQLI_MAX_TOKENS) {
         return NULL;
     }
     return &(sql_state->tokenvec[i]);
@@ -2203,7 +2248,7 @@ int libinjection_is_sqli(struct libinjection_sqli_state * sql_state)
     size_t slen = sql_state->slen;
 
     /*
-     * no input? not sqli
+     * no input? not SQLi
      */
     if (slen == 0) {
         return FALSE;
@@ -2263,3 +2308,18 @@ int libinjection_is_sqli(struct libinjection_sqli_state * sql_state)
      */
     return FALSE;
 }
+
+int libinjection_sqli(const char* input, size_t slen, char fingerprint[])
+{
+    int issqli;
+    struct libinjection_sqli_state state;
+
+    libinjection_sqli_init(&state, input, slen, 0);
+    issqli = libinjection_is_sqli(&state);
+    if (issqli) {
+        strcpy(fingerprint, state.fingerprint);
+    } else {
+        fingerprint[0] = '\0';
+    }
+    return issqli;
+}
