@@ -70,6 +70,7 @@ int json_add_argument(modsec_rec *msr, const char *value, unsigned length)
         return 0;
     }
 
+    arg->marked_for_sanitization = 0;
     apr_table_addn(msr->arguments,
         log_escape_nq_ex(msr->mp, arg->name, arg->name_len), (void *) arg);
 
@@ -402,7 +403,6 @@ int json_complete(modsec_rec *msr, char **error_msg) {
     assert(msr != NULL);
     assert(msr->json != NULL);
     assert(error_msg != NULL);
-    char *json_data = (char *) NULL;
 
     *error_msg = NULL;
 
