@@ -20,7 +20,7 @@
  */
 static apr_status_t msc_pcre_cleanup(msc_regex_t *regex) {
     if (regex != NULL) {
-#ifdef WITH_PCRE2
+#ifndef WITH_PCRE
         if (regex->match_context != NULL) {
             pcre2_match_context_free(regex->match_context);
             regex->match_context = NULL;
@@ -55,7 +55,7 @@ static apr_status_t msc_pcre_cleanup(msc_regex_t *regex) {
 void *msc_pregcomp_ex(apr_pool_t *pool, const char *pattern, int options,
                       const char **_errptr, int *_erroffset,
                       int match_limit, int match_limit_recursion)
-#ifdef WITH_PCRE2
+#ifndef WITH_PCRE
 {
     msc_regex_t *regex = NULL;
     PCRE2_SPTR pcre2_pattern;
@@ -77,6 +77,14 @@ void *msc_pregcomp_ex(apr_pool_t *pool, const char *pattern, int options,
         if (_erroffset != NULL) {
             *_erroffset = (int)error_offset;
         }
+        PCRE2_UCHAR buffer[256];
+        // Get the error message from the error code
+        pcre2_get_error_message(error_number, buffer, sizeof(buffer));
+#ifdef DEBUG_CONF
+        * _errptr = apr_psprintf(pool, "%s - pattern = %s", buffer, pattern);
+#else
+        * _errptr = apr_pstrdup(pool, buffer);
+#endif
         return NULL;
     }
 
@@ -231,7 +239,7 @@ int msc_regexec_ex(msc_regex_t *regex, const char *s, unsigned int slen,
     if (error_msg == NULL) return -1000; /* To differentiate from PCRE as it already uses -1. */
     *error_msg = NULL;
 
-#ifdef WITH_PCRE2
+#ifndef WITH_PCRE
     {
         PCRE2_SPTR pcre2_s;
         int pcre2_ret;
@@ -311,7 +319,7 @@ int msc_regexec(msc_regex_t *regex, const char *s, unsigned int slen,
  */
 int msc_fullinfo(msc_regex_t *regex, int what, void *where)
 {
-#ifdef WITH_PCRE2
+#ifndef WITH_PCRE
     return pcre2_pattern_info(regex->re, (uint32_t)what, where);
 #else
     return pcre_fullinfo(regex->re, regex->pe, what, where);
