@@ -25,6 +25,7 @@
 void msre_engine_reqbody_processor_register(msre_engine *engine,
     const char *name, void *fn_init, void *fn_process, void *fn_complete)
 {
+    assert(engine != NULL);
     msre_reqbody_processor_metadata *metadata = 
         (msre_reqbody_processor_metadata *)apr_pcalloc(engine->mp,
             sizeof(msre_reqbody_processor_metadata));
@@ -41,6 +42,8 @@ void msre_engine_reqbody_processor_register(msre_engine *engine,
  * Prepare to accept the request body (part 2).
  */
 static apr_status_t modsecurity_request_body_start_init(modsec_rec *msr, char **error_msg) {
+    assert(msr != NULL);
+    assert(error_msg != NULL);
     *error_msg = NULL;
 
     if(msr->msc_reqbody_storage == MSC_REQBODY_MEMORY) {
@@ -80,6 +83,8 @@ static apr_status_t modsecurity_request_body_start_init(modsec_rec *msr, char **
  * Prepare to accept the request body (part 1).
  */
 apr_status_t modsecurity_request_body_start(modsec_rec *msr, char **error_msg) {
+    assert(msr != NULL);
+    assert(error_msg != NULL);
     *error_msg = NULL;
     msr->msc_reqbody_length = 0;
     msr->stream_input_length = 0;
@@ -161,6 +166,8 @@ apr_status_t modsecurity_request_body_start(modsec_rec *msr, char **error_msg) {
 static apr_status_t modsecurity_request_body_store_disk(modsec_rec *msr,
     const char *data, apr_size_t length, char **error_msg)
 {
+    assert(msr != NULL);
+    assert(error_msg != NULL);
     apr_size_t i;
 
     *error_msg = NULL;
@@ -181,6 +188,8 @@ static apr_status_t modsecurity_request_body_store_disk(modsec_rec *msr,
 static apr_status_t modsecurity_request_body_store_memory(modsec_rec *msr,
     const char *data, apr_size_t length, char **error_msg)
 {
+    assert(msr != NULL);
+    assert(error_msg != NULL);
     *error_msg = NULL;
 
     /* Would storing this chunk mean going over the limit? */
@@ -309,6 +318,8 @@ static apr_status_t modsecurity_request_body_store_memory(modsec_rec *msr,
 apr_status_t modsecurity_request_body_store(modsec_rec *msr,
     const char *data, apr_size_t length, char **error_msg)
 {
+    assert(msr != NULL);
+    assert(error_msg != NULL);
     *error_msg = NULL;
 
     /* If we have a processor for this request body send
@@ -428,6 +439,9 @@ apr_status_t modsecurity_request_body_store(modsec_rec *msr,
 }
 
 apr_status_t modsecurity_request_body_to_stream(modsec_rec *msr, const char *buffer, int buflen, char **error_msg) {
+    assert(msr != NULL);
+    assert(error_msg != NULL);
+    assert(buffer != NULL || buflen == 0);
 #ifndef MSC_LARGE_STREAM_INPUT
     char *stream_input_body = NULL;
     char *data = NULL;
@@ -449,8 +463,8 @@ apr_status_t modsecurity_request_body_to_stream(modsec_rec *msr, const char *buf
         if(data == NULL)
             return -1;
 
-        memset(data, 0, msr->stream_input_length + 1 - buflen);
         memcpy(data, msr->stream_input_data, msr->stream_input_length - buflen);
+        data[msr->stream_input_length - buflen] = '\0';
 
         stream_input_body = (char *)realloc(msr->stream_input_data, msr->stream_input_length + 1);
 
@@ -458,28 +472,21 @@ apr_status_t modsecurity_request_body_to_stream(modsec_rec *msr, const char *buf
     }
 
     if (msr->stream_input_data == NULL) {
-        if(data)    {
-            free(data);
-            data = NULL;
-        }
+        if (data) free(data);
         *error_msg = apr_psprintf(msr->mp, "Unable to allocate memory to hold request body on stream. Asked for %" APR_SIZE_T_FMT " bytes.",
                 msr->stream_input_length + 1);
         return -1;
     }
 
-    memset(msr->stream_input_data, 0, msr->stream_input_length+1);
-
     if(first_pkt)   {
         memcpy(msr->stream_input_data, buffer, msr->stream_input_length);
     } else {
         memcpy(msr->stream_input_data, data, msr->stream_input_length - buflen);
         memcpy(msr->stream_input_data+(msr->stream_input_length - buflen), buffer, buflen);
     }
+    msr->stream_input_data[msr->stream_input_length] = '\0';
 
-    if(data)    {
-        free(data);
-        data = NULL;
-    }
+    if (data) free(data);
 #else
     if (msr->stream_input_data == NULL)  {
         // Is the request body length known beforehand? (requests that are not Transfer-Encoding: chunked)
@@ -541,6 +548,8 @@ apr_status_t modsecurity_request_body_to_stream(modsec_rec *msr, const char *buf
  * Replace a bunch of chunks holding a request body with a single large chunk.
  */
 static apr_status_t modsecurity_request_body_end_raw(modsec_rec *msr, char **error_msg) {
+    assert(msr != NULL);
+    assert(error_msg != NULL);
     msc_data_chunk **chunks, *one_chunk;
     char *d;
     int i, sofar;
@@ -614,6 +623,8 @@ static apr_status_t modsecurity_request_body_end_raw(modsec_rec *msr, char **err
  *
  */
 static apr_status_t modsecurity_request_body_end_urlencoded(modsec_rec *msr, char **error_msg) {
+    assert(msr != NULL);
+    assert(error_msg != NULL);
     int invalid_count = 0;
 
     *error_msg = NULL;
@@ -643,6 +654,8 @@ static apr_status_t modsecurity_request_body_end_urlencoded(modsec_rec *msr, cha
  * Stops receiving the request body.
  */
 apr_status_t modsecurity_request_body_end(modsec_rec *msr, char **error_msg) {
+    assert(msr != NULL);
+    assert(error_msg != NULL);
     *error_msg = NULL;
 
     /* Close open file descriptors, if any. */
@@ -753,6 +766,8 @@ apr_status_t modsecurity_request_body_end(modsec_rec *msr, char **error_msg) {
  * Prepares to forward the request body.
  */
 apr_status_t modsecurity_request_body_retrieve_start(modsec_rec *msr, char **error_msg) {
+    assert(msr != NULL);
+    assert(error_msg != NULL);
     *error_msg = NULL;
 
     if (msr->msc_reqbody_storage == MSC_REQBODY_MEMORY) {
@@ -799,6 +814,7 @@ apr_status_t modsecurity_request_body_retrieve_start(modsec_rec *msr, char **err
  *
  */
 apr_status_t modsecurity_request_body_retrieve_end(modsec_rec *msr) {
+    assert(msr != NULL);
     if (msr->msc_reqbody_storage == MSC_REQBODY_DISK) {
         if (msr->msc_reqbody_fd > 0) {
             close(msr->msc_reqbody_fd);
@@ -821,6 +837,8 @@ apr_status_t modsecurity_request_body_retrieve_end(modsec_rec *msr) {
 apr_status_t modsecurity_request_body_retrieve(modsec_rec *msr,
     msc_data_chunk **chunk, long int nbytes, char **error_msg)
 {
+    assert(msr != NULL);
+    assert(error_msg != NULL);
     msc_data_chunk **chunks;
 
     *error_msg = NULL;
@@ -922,6 +940,8 @@ apr_status_t modsecurity_request_body_retrieve(modsec_rec *msr,
  *
  */
 apr_status_t modsecurity_request_body_clear(modsec_rec *msr, char **error_msg) {
+    assert(msr != NULL);
+    assert(error_msg != NULL);
     *error_msg = NULL;
 
     /* Release memory we used to store request body data. */
