@@ -40,6 +40,10 @@
 
 #include "api.h"
 
+#ifdef WIN32
+#include "msc_status_engine.h"
+#endif
+
 extern void *modsecLogObj;
 extern void (*modsecLogHook)(void *obj, int level, char *str);
 extern int (*modsecDropAction)(request_rec *r);
@@ -75,6 +79,7 @@ DECLARE_HOOK(void,insert_filter,(request_rec *r))
 DECLARE_HOOK(void,insert_error_filter,(request_rec *r))
 
 char *sa_name = "standalone";
+const char *sa_name_argv[] = { "standalone", NULL };
 server_rec *server;
 apr_pool_t *pool = NULL;
 
@@ -135,7 +140,7 @@ server_rec *modsecInit()    {
     server->port = 80;
     server->process = apr_palloc(pool, sizeof(process_rec));
     server->process->argc = 1;
-    server->process->argv = &sa_name;
+    server->process->argv = sa_name_argv;
     server->process->pconf = pool;
     server->process->pool = pool;
     server->process->short_name = sa_name;
@@ -223,7 +228,6 @@ apr_status_t ap_http_in_filter(ap_filter_t *f, apr_bucket_brigade *bb_out,
 }
 
 apr_status_t ap_http_out_filter(ap_filter_t *f, apr_bucket_brigade *b)  {
-    apr_status_t rc;
     apr_bucket_brigade *bb_out = (apr_bucket_brigade *)f->ctx;
 
     APR_BRIGADE_CONCAT(bb_out, b);
@@ -266,7 +270,7 @@ const char *modsecProcessConfig(directory_config *config, const char *file, cons
 	incpath = file;
 
 	/* locate the start of the directories proper */
-	status = apr_filepath_root(&rootpath, &incpath, APR_FILEPATH_TRUENAME | APR_FILEPATH_NATIVE, pool);
+	status = apr_filepath_root(&rootpath, &incpath, APR_FILEPATH_TRUENAME | APR_FILEPATH_NATIVE, config->mp);
 
 	/* we allow APR_SUCCESS and APR_EINCOMPLETE */
 	if (APR_ERELATIVE == status) {
@@ -274,20 +278,20 @@ const char *modsecProcessConfig(directory_config *config, const char *file, cons
 
 		if(dir[li] != '/' && dir[li] != '\\')
 #ifdef	WIN32
-			file = apr_pstrcat(pool, dir, "\\", file, NULL);
+			file = apr_pstrcat(config->mp, dir, "\\", file, NULL);
 #else
-			file = apr_pstrcat(pool, dir, "/", file, NULL);
+			file = apr_pstrcat(config->mp, dir, "/", file, NULL);
 #endif
 		else
-			file = apr_pstrcat(pool, dir, file, NULL);
+			file = apr_pstrcat(config->mp, dir, file, NULL);
 	}
 	else if (APR_EBADPATH == status) {
-		return apr_pstrcat(pool, "Config file has a bad path, ", file, NULL);
+		return apr_pstrcat(config->mp, "Config file has a bad path, ", file, NULL);
 	}
 
-	apr_pool_create(&ptemp, pool);
+	apr_pool_create(&ptemp, config->mp);
 
-    err = process_command_config(server, config, pool, ptemp, file);
+    err = process_command_config(server, config, config->mp, ptemp, file);
 
     apr_pool_destroy(ptemp);
 
@@ -500,6 +504,16 @@ void modsecSetConfigForIISRequestBody(request_rec *r)
         msr->txcfg->stream_inbody_inspection = 1;
 }
 
+int modsecContextState(request_rec *r)
+{
+    modsec_rec *msr = retrieve_msr(r);
+
+    if(msr == NULL || msr->txcfg == NULL)
+        return NOT_SET;
+
+    return msr->txcfg->is_enabled;
+}
+
 int modsecIsRequestBodyAccessEnabled(request_rec *r)
 {
     modsec_rec *msr = retrieve_msr(r);
@@ -644,11 +658,20 @@ int modsecFinishRequest(request_rec *r) {
     // make sure you cleanup before calling apr_terminate()
     // otherwise double-free might occur, because of the request body pool cleanup function
     //
-    apr_pool_destroy(r->connection->pool);
+    apr_pool_destroy(r->pool);
 
     return DECLINED;
 }
 
+// destroy only the connection pool
+int modsecFinishConnection(conn_rec *c)
+{
+    apr_pool_destroy(c->pool);
+
+    return 0;
+}
+
+
 void modsecSetLogHook(void *obj, void (*hook)(void *obj, int level, char *str)) {
     modsecLogObj = obj;
     modsecLogHook = hook;
@@ -673,3 +696,55 @@ void modsecSetWriteResponse(apr_status_t (*func)(request_rec *r, char *buf, unsi
 void modsecSetDropAction(int (*func)(request_rec *r)) {
     modsecDropAction = func;
 }
+
+/*
+ * Case SecServerSignature was used, this function returns the banner that
+ * should be used, otherwise it returns NULL.
+ */
+const char *modsecIsServerSignatureAvailale(void) {
+    return new_server_signature;
+}
+
+#ifdef VERSION_IIS
+void modsecStatusEngineCall()
+{
+    if (status_engine_state != STATUS_ENGINE_DISABLED) {
+        msc_status_engine_call();
+    }
+    else {
+        ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, NULL,
+            "Status engine is currently disabled, enable it by set " \
+            "SecStatusEngine to On.\n");
+    }
+}
+
+void modsecReportRemoteLoadedRules()
+{
+#ifdef WITH_REMOTE_RULES
+    if (remote_rules_server != NULL)
+    {
+        if (remote_rules_server->amount_of_rules == 1)
+        {
+            ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, NULL,
+                "ModSecurity: Loaded %d rule from: '%s'.",
+                remote_rules_server->amount_of_rules,
+                remote_rules_server->uri);
+        }
+        else
+        {
+            ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, NULL,
+                "ModSecurity: Loaded %d rules from: '%s'.",
+                remote_rules_server->amount_of_rules,
+                remote_rules_server->uri);
+        }
+    }
+#endif
+    if (remote_rules_fail_message != NULL)
+    {
+        ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, NULL, "ModSecurity: " \
+            "Problems loading external resources: %s",
+            remote_rules_fail_message);
+    }
+
+}
+#endif
