@@ -58,6 +58,7 @@ void modsecInitProcess();
 
 conn_rec *modsecNewConnection();
 void modsecProcessConnection(conn_rec *c);
+int modsecFinishConnection(conn_rec *c);
 
 request_rec *modsecNewRequest(conn_rec *connection, directory_config *config);
 
@@ -112,8 +113,17 @@ void modsecSetDropAction(int (*func)(request_rec *r));
 int modsecIsResponseBodyAccessEnabled(request_rec *r);
 int modsecIsRequestBodyAccessEnabled(request_rec *r);
 
+int modsecContextState(request_rec *r);
+
 void modsecSetConfigForIISRequestBody(request_rec *r);
 
+const char *modsecIsServerSignatureAvailale(void);
+
+#ifdef VERSION_IIS
+void modsecStatusEngineCall(void);
+void modsecReportRemoteLoadedRules(void);
+#endif
+
 #ifdef __cplusplus
 }
 #endif
